@@ -1,68 +1,98 @@
+import traceback
 from collections import defaultdict
-from datetime import date, datetime
-from typing import Callable, Type
+from datetime import date, datetime, timedelta
+from typing import Callable, Type, Dict, List, Optional
+from functools import partial
 
 import numpy as np
 from pandas import DataFrame
 import plotly.graph_objects as go
 from plotly.subplots import make_subplots
 
-from vnpy.trader.constant import (Direction, Offset, Exchange,
-                                  Interval, Status)
+from vnpy.trader.constant import (
+    Direction,
+    Offset,
+    Exchange,
+    Interval,
+    Status
+)
 from vnpy.trader.object import TradeData, BarData, TickData
+from vnpy.trader.optimize import (
+    OptimizationSetting,
+    check_optimization_setting,
+    run_bf_optimization,
+    run_ga_optimization
+)
 
 from .template import SpreadStrategyTemplate, SpreadAlgoTemplate
-from .base import SpreadData, BacktestingMode, load_bar_data, load_tick_data
+from .base import (
+    SpreadData,
+    BacktestingMode,
+    load_bar_data,
+    load_tick_data,
+    EngineType
+)
+
+
+INTERVAL_DELTA_MAP: Dict[Interval, timedelta] = {
+    Interval.TICK: timedelta(milliseconds=1),
+    Interval.MINUTE: timedelta(minutes=1),
+    Interval.HOUR: timedelta(hours=1),
+    Interval.DAILY: timedelta(days=1),
+}
 
 
 class BacktestingEngine:
     """"""
 
-    gateway_name = "BACKTESTING"
+    engine_type: EngineType = EngineType.BACKTESTING
+    gateway_name: str = "BACKTESTING"
 
-    def __init__(self):
+    def __init__(self) -> None:
         """"""
         self.spread: SpreadData = None
 
-        self.start = None
-        self.end = None
-        self.rate = 0
-        self.slippage = 0
-        self.size = 1
-        self.pricetick = 0
-        self.capital = 1_000_000
-        self.mode = BacktestingMode.BAR
+        self.start: datetime = None
+        self.end: datetime = None
+        self.rate: float = 0
+        self.slippage: float = 0
+        self.size: float = 1
+        self.pricetick: float = 0
+        self.capital: int = 1_000_000
+        self.risk_free: float = 0
+        self.annual_days: int = 240
+        self.mode: BacktestingMode = BacktestingMode.BAR
 
         self.strategy_class: Type[SpreadStrategyTemplate] = None
         self.strategy: SpreadStrategyTemplate = None
         self.tick: TickData = None
         self.bar: BarData = None
-        self.datetime = None
+        self.datetime: datetime = None
 
-        self.interval = None
-        self.days = 0
-        self.callback = None
-        self.history_data = []
+        self.interval: Interval = None
+        self.days: int = 0
+        self.callback: Callable = None
+        self.history_data: list = []
 
-        self.algo_count = 0
-        self.algos = {}
-        self.active_algos = {}
+        self.algo_count: int = 0
+        self.algos: Dict[str, SpreadAlgoTemplate] = {}
+        self.active_algos: Dict[str, SpreadAlgoTemplate] = {}
 
-        self.trade_count = 0
-        self.trades = {}
+        self.trade_count: int = 0
+        self.trades: Dict[str, TradeData] = {}
 
-        self.logs = []
+        self.logs: list = []
 
-        self.daily_results = {}
-        self.daily_df = None
+        self.daily_results: Dict[date, DailyResult] = {}
+        self.daily_df: DataFrame = None
 
-    def output(self, msg):
+    def output(self, msg) -> None:
         """
         Output message of backtesting engine.
         """
         print(f"{datetime.now()}\t{msg}")
 
-    def clear_data(self):
+    def clear_data(self) -> None:
         """
         Clear all data of last backtesting.
         """
@@ -70,6 +100,7 @@ class BacktestingEngine:
         self.tick = None
         self.bar = None
         self.datetime = None
+        self.spread.net_pos = 0
 
         self.algo_count = 0
         self.algos.clear()
@@ -91,9 +122,11 @@ class BacktestingEngine:
         size: float,
         pricetick: float,
         capital: int = 0,
+        risk_free: float = 0,
+        annual_days: int = 240,
         end: datetime = None,
         mode: BacktestingMode = BacktestingMode.BAR
-    ):
+    ) -> None:
         """"""
         self.spread = spread
         self.interval = Interval(interval)
@@ -103,10 +136,12 @@ class BacktestingEngine:
         self.pricetick = pricetick
         self.start = start
         self.capital = capital
+        self.risk_free = risk_free
+        self.annual_days = annual_days
         self.end = end
         self.mode = mode
 
-    def add_strategy(self, strategy_class: type, setting: dict):
+    def add_strategy(self, strategy_class: type, setting: dict) -> None:
         """"""
         self.strategy_class = strategy_class
 
@@ -117,7 +152,7 @@ class BacktestingEngine:
             setting
         )
 
-    def load_data(self):
+    def load_data(self) -> None:
         """"""
         self.output("开始加载历史数据")
 
@@ -130,11 +165,12 @@ class BacktestingEngine:
 
         if self.mode == BacktestingMode.BAR:
             self.history_data = load_bar_data(
-                self.spread,
-                self.interval,
-                self.start,
-                self.end,
-                self.pricetick
+                spread=self.spread,
+                interval=self.interval,
+                start=self.start,
+                end=self.end,
+                pricetick=self.pricetick,
+                backtesting=True
             )
         else:
             self.history_data = load_tick_data(
@@ -145,7 +181,7 @@ class BacktestingEngine:
 
         self.output(f"历史数据加载完成，数据量：{len(self.history_data)}")
 
-    def run_backtesting(self):
+    def run_backtesting(self) -> None:
         """"""
         if self.mode == BacktestingMode.BAR:
             func = self.new_bar
@@ -153,20 +189,6 @@ class BacktestingEngine:
             func = self.new_tick
 
         self.strategy.on_init()
-
-        # Use the first [days] of history data for initializing strategy
-        day_count = 0
-        ix = 0
-
-        for ix, data in enumerate(self.history_data):
-            if self.datetime and data.datetime.day != self.datetime.day:
-                day_count += 1
-                if day_count >= self.days:
-                    break
-
-            self.datetime = data.datetime
-            self.callback(data)
-
         self.strategy.inited = True
         self.output("策略初始化完成")
 
@@ -174,23 +196,26 @@ class BacktestingEngine:
         self.strategy.trading = True
         self.output("开始回放历史数据")
 
-        # Use the rest of history data for running backtesting
-        for data in self.history_data[ix:]:
-            func(data)
+        for data in self.history_data:
+            try:
+                func(data)
+            except Exception:
+                self.output("触发异常，回测终止")
+                self.output(traceback.format_exc())
+                return
 
         self.output("历史数据回放结束")
 
-    def calculate_result(self):
+    def calculate_result(self) -> DataFrame:
         """"""
         self.output("开始计算逐日盯市盈亏")
 
         if not self.trades:
-            self.output("成交记录为空，无法计算")
-            return
+            self.output("回测成交记录为空")
 
         # Add trade data into daily reuslt.
         for trade in self.trades.values():
-            d = trade.datetime.date()
+            d: date = trade.datetime.date()
             daily_result = self.daily_results[d]
             daily_result.add_trade(trade)
 
@@ -211,54 +236,58 @@ class BacktestingEngine:
             start_pos = daily_result.end_pos
 
         # Generate dataframe
-        results = defaultdict(list)
+        results: defaultdict = defaultdict(list)
 
         for daily_result in self.daily_results.values():
             for key, value in daily_result.__dict__.items():
                 results[key].append(value)
 
-        self.daily_df = DataFrame.from_dict(results).set_index("date")
+        if results:
+            self.daily_df: DataFrame = DataFrame.from_dict(results).set_index("date")
 
         self.output("逐日盯市盈亏计算完成")
         return self.daily_df
 
-    def calculate_statistics(self, df: DataFrame = None, output=True):
+    def calculate_statistics(self, df: DataFrame = None, output=True) -> dict:
         """"""
         self.output("开始计算策略统计指标")
 
         # Check DataFrame input exterior
         if df is None:
-            df = self.daily_df
+            df: DataFrame = self.daily_df
+
+        # Init all statistics default value
+        start_date: str = ""
+        end_date: str = ""
+        total_days: int = 0
+        profit_days: int = 0
+        loss_days: int = 0
+        end_balance: float = 0
+        max_drawdown: float = 0
+        max_ddpercent: float = 0
+        max_drawdown_duration: int = 0
+        total_net_pnl: float = 0
+        daily_net_pnl: float = 0
+        total_commission: float = 0
+        daily_commission: float = 0
+        total_slippage: float = 0
+        daily_slippage: float = 0
+        total_turnover: float = 0
+        daily_turnover: float = 0
+        total_trade_count: int = 0
+        daily_trade_count: int = 0
+        total_return: float = 0
+        annual_return: float = 0
+        daily_return: float = 0
+        return_std: float = 0
+        sharpe_ratio: float = 0
+        return_drawdown_ratio: float = 0
+
+        # Check if balance is always positive
+        positive_balance: bool = False
 
         # Check for init DataFrame
-        if df is None:
-            # Set all statistics to 0 if no trade.
-            start_date = ""
-            end_date = ""
-            total_days = 0
-            profit_days = 0
-            loss_days = 0
-            end_balance = 0
-            max_drawdown = 0
-            max_ddpercent = 0
-            max_drawdown_duration = 0
-            total_net_pnl = 0
-            daily_net_pnl = 0
-            total_commission = 0
-            daily_commission = 0
-            total_slippage = 0
-            daily_slippage = 0
-            total_turnover = 0
-            daily_turnover = 0
-            total_trade_count = 0
-            daily_trade_count = 0
-            total_return = 0
-            annual_return = 0
-            daily_return = 0
-            return_std = 0
-            sharpe_ratio = 0
-            return_drawdown_ratio = 0
-        else:
+        if df is not None:
             # Calculate balance related time series data
             df["balance"] = df["net_pnl"].cumsum() + self.capital
             df["return"] = np.log(df["balance"] / df["balance"].shift(1)).fillna(0)
@@ -269,47 +298,54 @@ class BacktestingEngine:
             df["drawdown"] = df["balance"] - df["highlevel"]
             df["ddpercent"] = df["drawdown"] / df["highlevel"] * 100
 
+            # All balance value needs to be positive
+            positive_balance = (df["balance"] > 0).all()
+            if not positive_balance:
+                self.output("回测中出现爆仓（资金小于等于0），无法计算策略统计指标")
+
+        if positive_balance:
             # Calculate statistics value
             start_date = df.index[0]
             end_date = df.index[-1]
 
-            total_days = len(df)
-            profit_days = len(df[df["net_pnl"] > 0])
-            loss_days = len(df[df["net_pnl"] < 0])
+            total_days: int = len(df)
+            profit_days: int = len(df[df["net_pnl"] > 0])
+            loss_days: int = len(df[df["net_pnl"] < 0])
 
-            end_balance = df["balance"].iloc[-1]
-            max_drawdown = df["drawdown"].min()
-            max_ddpercent = df["ddpercent"].min()
-            max_drawdown_end = df["drawdown"].idxmin()
-            max_drawdown_start = df["balance"][:max_drawdown_end].idxmax()
-            max_drawdown_duration = (max_drawdown_end - max_drawdown_start).days
+            end_balance: float = df["balance"].iloc[-1]
+            max_drawdown: float = df["drawdown"].min()
+            max_ddpercent: float = df["ddpercent"].min()
+            max_drawdown_end: float = df["drawdown"].idxmin()
+            max_drawdown_start: float = df["balance"][:max_drawdown_end].idxmax()
+            max_drawdown_duration: int = (max_drawdown_end - max_drawdown_start).days
 
-            total_net_pnl = df["net_pnl"].sum()
-            daily_net_pnl = total_net_pnl / total_days
+            total_net_pnl: float = df["net_pnl"].sum()
+            daily_net_pnl: float = total_net_pnl / total_days
 
-            total_commission = df["commission"].sum()
-            daily_commission = total_commission / total_days
+            total_commission: float = df["commission"].sum()
+            daily_commission: float = total_commission / total_days
 
-            total_slippage = df["slippage"].sum()
-            daily_slippage = total_slippage / total_days
+            total_slippage: float = df["slippage"].sum()
+            daily_slippage: float = total_slippage / total_days
 
-            total_turnover = df["turnover"].sum()
-            daily_turnover = total_turnover / total_days
+            total_turnover: float = df["turnover"].sum()
+            daily_turnover: float = total_turnover / total_days
 
-            total_trade_count = df["trade_count"].sum()
-            daily_trade_count = total_trade_count / total_days
+            total_trade_count: int = df["trade_count"].sum()
+            daily_trade_count: int = total_trade_count / total_days
 
-            total_return = (end_balance / self.capital - 1) * 100
-            annual_return = total_return / total_days * 240
-            daily_return = df["return"].mean() * 100
-            return_std = df["return"].std() * 100
+            total_return: float = (end_balance / self.capital - 1) * 100
+            annual_return: float = total_return / total_days * self.annual_days
+            daily_return: float = df["return"].mean() * 100
+            return_std: float = df["return"].std() * 100
 
             if return_std:
-                sharpe_ratio = daily_return / return_std * np.sqrt(240)
+                daily_risk_free: float = self.risk_free / self.annual_days
+                sharpe_ratio: float = (daily_return - daily_risk_free) / return_std * np.sqrt(self.annual_days)
             else:
-                sharpe_ratio = 0
+                sharpe_ratio: float = 0
 
-            return_drawdown_ratio = -total_return / max_ddpercent
+            return_drawdown_ratio: float = -total_return / max_ddpercent
 
         # Output
         if output:
@@ -347,7 +383,7 @@ class BacktestingEngine:
             self.output(f"Sharpe Ratio：\t{sharpe_ratio:,.2f}")
             self.output(f"收益回撤比：\t{return_drawdown_ratio:,.2f}")
 
-        statistics = {
+        statistics: dict = {
             "start_date": start_date,
             "end_date": end_date,
             "total_days": total_days,
@@ -378,11 +414,11 @@ class BacktestingEngine:
 
         return statistics
 
-    def show_chart(self, df: DataFrame = None):
+    def show_chart(self, df: DataFrame = None) -> None:
         """"""
         # Check DataFrame input exterior
         if df is None:
-            df = self.daily_df
+            df: DataFrame = self.daily_df
 
         # Check for init DataFrame
         if df is None:
@@ -420,17 +456,73 @@ class BacktestingEngine:
         fig.update_layout(height=1000, width=1000)
         fig.show()
 
-    def update_daily_close(self, price: float):
+    def run_bf_optimization(
+        self,
+        optimization_setting: OptimizationSetting,
+        output=True,
+        max_workers: int = None
+    ) -> list:
         """"""
-        d = self.datetime.date()
+        if not check_optimization_setting(optimization_setting):
+            return
 
-        daily_result = self.daily_results.get(d, None)
+        evaluate_func: callable = wrap_evaluate(self, optimization_setting.target_name)
+        results: list = run_bf_optimization(
+            evaluate_func,
+            optimization_setting,
+            get_target_value,
+            max_workers=max_workers,
+            output=self.output,
+        )
+
+        if output:
+            for result in results:
+                msg: str = f"参数：{result[0]}, 目标：{result[1]}"
+                self.output(msg)
+
+        return results
+
+    run_optimization = run_bf_optimization
+
+    def run_ga_optimization(
+        self,
+        optimization_setting: OptimizationSetting,
+        output=True,
+        max_workers: int = None,
+        ngen_size: int = 30
+    ) -> list:
+        """"""
+        if not check_optimization_setting(optimization_setting):
+            return
+
+        evaluate_func: callable = wrap_evaluate(self, optimization_setting.target_name)
+        results: list = run_ga_optimization(
+            evaluate_func,
+            optimization_setting,
+            get_target_value,
+            max_workers=max_workers,
+            ngen_size=ngen_size,
+            output=self.output
+        )
+
+        if output:
+            for result in results:
+                msg: str = f"参数：{result[0]}, 目标：{result[1]}"
+                self.output(msg)
+
+        return results
+
+    def update_daily_close(self, price: float) -> None:
+        """"""
+        d: date = self.datetime.date()
+
+        daily_result: Optional[DailyResult] = self.daily_results.get(d, None)
         if daily_result:
             daily_result.close_price = price
         else:
             self.daily_results[d] = DailyResult(d, price)
 
-    def new_bar(self, bar: BarData):
+    def new_bar(self, bar: BarData) -> None:
         """"""
         self.bar = bar
         self.datetime = bar.datetime
@@ -440,7 +532,7 @@ class BacktestingEngine:
 
         self.update_daily_close(bar.close_price)
 
-    def new_tick(self, tick: TickData):
+    def new_tick(self, tick: TickData) -> None:
         """"""
         self.tick = tick
         self.datetime = tick.datetime
@@ -456,7 +548,7 @@ class BacktestingEngine:
 
         self.update_daily_close(tick.last_price)
 
-    def cross_algo(self):
+    def cross_algo(self) -> None:
         """
         Cross limit order with last bar/tick data.
         """
@@ -469,12 +561,12 @@ class BacktestingEngine:
 
         for algo in list(self.active_algos.values()):
             # Check whether limit orders can be filled.
-            long_cross = (
+            long_cross: bool = (
                 algo.direction == Direction.LONG
                 and algo.price >= long_cross_price
             )
 
-            short_cross = (
+            short_cross: bool = (
                 algo.direction == Direction.SHORT
                 and algo.price <= short_cross_price
             )
@@ -483,7 +575,9 @@ class BacktestingEngine:
                 continue
 
             # Push order udpate with status "all traded" (filled).
-            algo.traded = algo.volume
+            algo.traded = algo.target
+            algo.traded_volume = algo.volume
+            algo.traded_price = algo.price
             algo.status = Status.ALLTRADED
             self.strategy.update_spread_algo(algo)
 
@@ -499,7 +593,7 @@ class BacktestingEngine:
                 trade_price = short_cross_price
                 pos_change = -algo.volume
 
-            trade = TradeData(
+            trade: TradeData = TradeData(
                 symbol=self.spread.name,
                 exchange=Exchange.LOCAL,
                 orderid=algo.algoid,
@@ -523,15 +617,44 @@ class BacktestingEngine:
 
     def load_bar(
         self, spread: SpreadData, days: int, interval: Interval, callback: Callable
-    ):
+    ) -> None:
         """"""
-        self.days = days
         self.callback = callback
 
-    def load_tick(self, spread: SpreadData, days: int, callback: Callable):
+        init_end = self.start - INTERVAL_DELTA_MAP[interval]
+        init_start = self.start - timedelta(days=days)
+
+        bars: List[BarData] = load_bar_data(
+            spread=self.spread,
+            interval=self.interval,
+            start=init_start,
+            end=init_end,
+            pricetick=self.pricetick,
+            backtesting=True
+        )
+
+        for bar in bars:
+            callback(bar)
+
+        return bars
+
+    def load_tick(self, spread: SpreadData, days: int, callback: Callable) -> None:
         """"""
         self.days = days
-        self.callback = callback
+
+        init_end = self.start - INTERVAL_DELTA_MAP[Interval.TICK]
+        init_start = self.start - timedelta(days=days)
+
+        ticks: List[TickData] = load_tick_data(
+            self.spread,
+            init_start,
+            init_end
+        )
+
+        for tick in ticks:
+            callback(callback)
+
+        return ticks
 
     def start_algo(
         self,
@@ -547,9 +670,9 @@ class BacktestingEngine:
     ) -> str:
         """"""
         self.algo_count += 1
-        algoid = str(self.algo_count)
+        algoid: str = str(self.algo_count)
 
-        algo = SpreadAlgoTemplate(
+        algo: SpreadAlgoTemplate = SpreadAlgoTemplate(
             self,
             algoid,
             self.spread,
@@ -571,11 +694,11 @@ class BacktestingEngine:
         self,
         strategy: SpreadStrategyTemplate,
         algoid: str
-    ):
+    ) -> None:
         """"""
         if algoid not in self.active_algos:
             return
-        algo = self.active_algos.pop(algoid)
+        algo: SpreadAlgoTemplate = self.active_algos.pop(algoid)
 
         algo.status = Status.CANCELLED
         self.strategy.update_spread_algo(algo)
@@ -589,36 +712,42 @@ class BacktestingEngine:
         volume: float,
         stop: bool,
         lock: bool
-    ):
+    ) -> None:
         """"""
         pass
 
-    def cancel_order(self, strategy: SpreadStrategyTemplate, vt_orderid: str):
+    def cancel_order(self, strategy: SpreadStrategyTemplate, vt_orderid: str) -> None:
         """
         Cancel order by vt_orderid.
         """
         pass
 
-    def write_strategy_log(self, strategy: SpreadStrategyTemplate, msg: str):
+    def write_strategy_log(self, strategy: SpreadStrategyTemplate, msg: str) -> None:
         """
         Write log message.
         """
-        msg = f"{self.datetime}\t{msg}"
+        msg: str = f"{self.datetime}\t{msg}"
         self.logs.append(msg)
 
-    def send_email(self, msg: str, strategy: SpreadStrategyTemplate = None):
+    def send_email(self, msg: str, strategy: SpreadStrategyTemplate = None) -> None:
         """
         Send email to default receiver.
         """
         pass
 
-    def put_strategy_event(self, strategy: SpreadStrategyTemplate):
+    def get_engine_type(self) -> EngineType:
+        """
+        Return engine type.
+        """
+        return self.engine_type
+
+    def put_strategy_event(self, strategy: SpreadStrategyTemplate) -> None:
         """
         Put an event to update strategy status.
         """
         pass
 
-    def write_algo_log(self, algo: SpreadAlgoTemplate, msg: str):
+    def write_algo_log(self, algo: SpreadAlgoTemplate, msg: str) -> None:
         """"""
         pass
 
@@ -626,28 +755,28 @@ class BacktestingEngine:
 class DailyResult:
     """"""
 
-    def __init__(self, date: date, close_price: float):
+    def __init__(self, date: date, close_price: float) -> None:
         """"""
-        self.date = date
-        self.close_price = close_price
-        self.pre_close = 0
+        self.date: date = date
+        self.close_price: float = close_price
+        self.pre_close: float = 0
 
-        self.trades = []
-        self.trade_count = 0
+        self.trades: List[TradeData] = []
+        self.trade_count: int = 0
 
         self.start_pos = 0
         self.end_pos = 0
 
-        self.turnover = 0
-        self.commission = 0
-        self.slippage = 0
+        self.turnover: float = 0
+        self.commission: float = 0
+        self.slippage: float = 0
 
-        self.trading_pnl = 0
-        self.holding_pnl = 0
-        self.total_pnl = 0
-        self.net_pnl = 0
+        self.trading_pnl: float = 0
+        self.holding_pnl: float = 0
+        self.total_pnl: float = 0
+        self.net_pnl: float = 0
 
-    def add_trade(self, trade: TradeData):
+    def add_trade(self, trade: TradeData) -> None:
         """"""
         self.trades.append(trade)
 
@@ -658,7 +787,7 @@ class DailyResult:
         size: int,
         rate: float,
         slippage: float
-    ):
+    ) -> None:
         """"""
         # If no pre_close provided on the first day,
         # use value 1 to avoid zero division error
@@ -684,7 +813,7 @@ class DailyResult:
 
             self.end_pos += pos_change
 
-            turnover = trade.volume * size * trade.value
+            turnover: float = trade.volume * size * trade.value
             self.trading_pnl += pos_change * \
                 (self.close_price - trade.price) * size
             self.slippage += trade.volume * size * slippage
@@ -695,3 +824,72 @@ class DailyResult:
         # Net pnl takes account of commission and slippage cost
         self.total_pnl = self.trading_pnl + self.holding_pnl
         self.net_pnl = self.total_pnl - self.commission - self.slippage
+
+
+def evaluate(
+    target_name: str,
+    strategy_class: SpreadStrategyTemplate,
+    spread: SpreadData,
+    interval: Interval,
+    start: datetime,
+    rate: float,
+    slippage: float,
+    size: float,
+    pricetick: float,
+    capital: int,
+    end: datetime,
+    setting: dict
+) -> tuple:
+    """
+    Function for running in multiprocessing.pool
+    """
+    engine: BacktestingEngine = BacktestingEngine()
+
+    engine.set_parameters(
+        spread=spread,
+        interval=interval,
+        start=start,
+        rate=rate,
+        slippage=slippage,
+        size=size,
+        pricetick=pricetick,
+        capital=capital,
+        end=end,
+    )
+
+    engine.add_strategy(strategy_class, setting)
+    engine.load_data()
+    engine.run_backtesting()
+    engine.calculate_result()
+    statistics: dict = engine.calculate_statistics(output=False)
+
+    target_value: float = statistics[target_name]
+    return (setting, target_value, statistics)
+
+
+def wrap_evaluate(engine: BacktestingEngine, target_name: str) -> callable:
+    """
+    Wrap evaluate function with given setting from backtesting engine.
+    """
+    func: callable = partial(
+        evaluate,
+        target_name,
+        engine.strategy_class,
+        engine.spread,
+        engine.interval,
+        engine.start,
+        engine.rate,
+        engine.slippage,
+        engine.size,
+        engine.pricetick,
+        engine.capital,
+        engine.end
+    )
+    return func
+
+
+def get_target_value(result: list) -> float:
+    """
+    Get target value for sorting optimization results.
+    """
+    return result[1]
