@@ -2,8 +2,11 @@
 Widget for spread trading.
 """
 
+from typing import Dict, List, Any
+
 from vnpy.event import EventEngine, Event
 from vnpy.trader.engine import MainEngine
+from vnpy.trader.object import LogData
 from vnpy.trader.constant import Direction
 from vnpy.trader.ui import QtWidgets, QtCore, QtGui
 from vnpy.trader.ui.widget import (
@@ -16,6 +19,7 @@ from vnpy.trader.ui.widget import (
 from ..engine import (
     SpreadEngine,
     SpreadStrategyEngine,
+    SpreadData,
     APP_NAME,
     EVENT_SPREAD_DATA,
     EVENT_SPREAD_POS,
@@ -28,64 +32,66 @@ from ..engine import (
 class SpreadManager(QtWidgets.QWidget):
     """"""
 
-    def __init__(self, main_engine: MainEngine, event_engine: EventEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         """"""
         super().__init__()
 
-        self.main_engine = main_engine
-        self.event_engine = event_engine
+        self.main_engine: MainEngine = main_engine
+        self.event_engine: EventEngine = event_engine
 
-        self.spread_engine = main_engine.get_engine(APP_NAME)
+        self.spread_engine: SpreadEngine = main_engine.get_engine(APP_NAME)
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setWindowTitle("价差交易")
 
-        self.algo_dialog = SpreadAlgoWidget(self.spread_engine)
-        algo_group = self.create_group("交易", self.algo_dialog)
+        self.algo_dialog: SpreadAlgoWidget = SpreadAlgoWidget(self.spread_engine)
+        algo_group: QtWidgets.QGroupBox = self.create_group("交易", self.algo_dialog)
         algo_group.setMaximumWidth(300)
 
-        self.data_monitor = SpreadDataMonitor(
+        self.data_monitor: SpreadDataMonitor = SpreadDataMonitor(
             self.main_engine,
             self.event_engine
         )
-        self.log_monitor = SpreadLogMonitor(
+        self.log_monitor: SpreadLogMonitor = SpreadLogMonitor(
             self.main_engine,
             self.event_engine
         )
-        self.algo_monitor = SpreadAlgoMonitor(
-            self.spread_engine
+        self.algo_monitor: SpreadAlgoMonitor = SpreadAlgoMonitor(
+            self.main_engine,
+            self.event_engine
         )
 
-        self.strategy_monitor = SpreadStrategyMonitor(
-            self.spread_engine
+        self.strategy_monitor: SpreadStrategyMonitor = SpreadStrategyMonitor(
+            self.main_engine,
+            self.event_engine
         )
 
-        grid = QtWidgets.QGridLayout()
+        grid: QtWidgets.QGridLayout = QtWidgets.QGridLayout()
         grid.addWidget(self.create_group("价差", self.data_monitor), 0, 0)
         grid.addWidget(self.create_group("日志", self.log_monitor), 1, 0)
         grid.addWidget(self.create_group("算法", self.algo_monitor), 0, 1)
         grid.addWidget(self.create_group("策略", self.strategy_monitor), 1, 1)
 
-        hbox = QtWidgets.QHBoxLayout()
+        hbox: QtWidgets.QHBoxLayout = QtWidgets.QHBoxLayout()
         hbox.addWidget(algo_group)
         hbox.addLayout(grid)
 
         self.setLayout(hbox)
 
-    def show(self):
+    def show(self) -> None:
         """"""
         self.spread_engine.start()
         self.algo_dialog.update_class_combo()
         self.showMaximized()
 
-    def create_group(self, title: str, widget: QtWidgets.QWidget):
+    def create_group(self, title: str, widget: QtWidgets.QWidget) -> QtWidgets.QGroupBox:
         """"""
-        group = QtWidgets.QGroupBox()
+        group: QtWidgets.QGroupBox = QtWidgets.QGroupBox()
 
-        vbox = QtWidgets.QVBoxLayout()
+        vbox: QtWidgets.QVBoxLayout = QtWidgets.QVBoxLayout()
         vbox.addWidget(widget)
 
         group.setLayout(vbox)
@@ -99,11 +105,11 @@ class SpreadDataMonitor(BaseMonitor):
     Monitor for spread data.
     """
 
-    event_type = EVENT_SPREAD_DATA
-    data_key = "name"
-    sorting = False
+    event_type: str = EVENT_SPREAD_DATA
+    data_key: str = "name"
+    sorting: bool = False
 
-    headers = {
+    headers: dict = {
         "name": {"display": "名称", "cell": BaseCell, "update": False},
         "bid_volume": {"display": "买量", "cell": BidCell, "update": True},
         "bid_price": {"display": "买价", "cell": BidCell, "update": True},
@@ -115,7 +121,7 @@ class SpreadDataMonitor(BaseMonitor):
         "trading_formula": {"display": "交易", "cell": BaseCell, "update": False},
     }
 
-    def register_event(self):
+    def register_event(self) -> None:
         """
         Register event handler into event engine.
         """
@@ -127,19 +133,19 @@ class SpreadLogMonitor(QtWidgets.QTextEdit):
     """
     Monitor for log data.
     """
-    signal = QtCore.pyqtSignal(Event)
+    signal: QtCore.Signal = QtCore.Signal(Event)
 
-    def __init__(self, main_engine: MainEngine, event_engine: EventEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         """"""
         super().__init__()
 
-        self.main_engine = main_engine
-        self.event_engine = event_engine
+        self.main_engine: MainEngine = main_engine
+        self.event_engine: EventEngine = event_engine
 
         self.init_ui()
         self.register_event()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setReadOnly(True)
 
@@ -149,10 +155,10 @@ class SpreadLogMonitor(QtWidgets.QTextEdit):
 
         self.event_engine.register(EVENT_SPREAD_LOG, self.signal.emit)
 
-    def process_log_event(self, event: Event):
+    def process_log_event(self, event: Event) -> None:
         """"""
-        log = event.data
-        msg = f"{log.time.strftime('%H:%M:%S')}\t{log.msg}"
+        log: LogData = event.data
+        msg: str = f"{log.time.strftime('%H:%M:%S')}\t{log.msg}"
         self.append(msg)
 
 
@@ -161,11 +167,11 @@ class SpreadAlgoMonitor(BaseMonitor):
     Monitor for algo status.
     """
 
-    event_type = EVENT_SPREAD_ALGO
-    data_key = "algoid"
-    sorting = False
+    event_type: str = EVENT_SPREAD_ALGO
+    data_key: str = "algoid"
+    sorting: bool = False
 
-    headers = {
+    headers: dict = {
         "algoid": {"display": "算法", "cell": BaseCell, "update": False},
         "spread_name": {"display": "价差", "cell": BaseCell, "update": False},
         "direction": {"display": "方向", "cell": DirectionCell, "update": False},
@@ -179,13 +185,13 @@ class SpreadAlgoMonitor(BaseMonitor):
         "status": {"display": "状态", "cell": EnumCell, "update": True},
     }
 
-    def __init__(self, spread_engine: SpreadEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         """"""
-        super().__init__(spread_engine.main_engine, spread_engine.event_engine)
+        super().__init__(main_engine, event_engine)
 
-        self.spread_engine = spread_engine
+        self.spread_engine: SpreadEngine = main_engine.get_engine(APP_NAME)
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """
         Connect signal.
         """
@@ -194,7 +200,7 @@ class SpreadAlgoMonitor(BaseMonitor):
         self.setToolTip("双击单元格停止算法")
         self.itemDoubleClicked.connect(self.stop_algo)
 
-    def stop_algo(self, cell):
+    def stop_algo(self, cell) -> None:
         """
         Stop algo if cell double clicked.
         """
@@ -205,7 +211,7 @@ class SpreadAlgoMonitor(BaseMonitor):
 class SpreadAlgoWidget(QtWidgets.QFrame):
     """"""
 
-    def __init__(self, spread_engine: SpreadEngine):
+    def __init__(self, spread_engine: SpreadEngine) -> None:
         """"""
         super().__init__()
 
@@ -214,64 +220,64 @@ class SpreadAlgoWidget(QtWidgets.QFrame):
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setWindowTitle("启动算法")
-        self.setFrameShape(self.Box)
+        self.setFrameShape(self.Shape.Box)
         self.setLineWidth(1)
 
-        self.name_line = QtWidgets.QLineEdit()
+        self.name_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
 
-        self.direction_combo = QtWidgets.QComboBox()
+        self.direction_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
         self.direction_combo.addItems(
             [Direction.LONG.value, Direction.SHORT.value]
         )
 
-        float_validator = QtGui.QDoubleValidator()
+        float_validator: QtGui.QDoubleValidator = QtGui.QDoubleValidator()
 
-        self.price_line = QtWidgets.QLineEdit()
+        self.price_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
         self.price_line.setValidator(float_validator)
 
-        self.volume_line = QtWidgets.QLineEdit()
+        self.volume_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
         self.volume_line.setValidator(float_validator)
 
-        int_validator = QtGui.QIntValidator()
+        int_validator: QtGui.QIntValidator = QtGui.QIntValidator()
 
-        self.payup_line = QtWidgets.QLineEdit()
+        self.payup_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
         self.payup_line.setValidator(int_validator)
 
-        self.interval_line = QtWidgets.QLineEdit()
+        self.interval_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
         self.interval_line.setValidator(int_validator)
 
-        button_start = QtWidgets.QPushButton("启动")
+        button_start: QtWidgets.QPushButton = QtWidgets.QPushButton("启动")
         button_start.clicked.connect(self.start_algo)
 
-        self.mode_combo = QtWidgets.QComboBox()
+        self.mode_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
         self.mode_combo.addItems(
             ["净仓", "锁仓"]
         )
 
-        self.class_combo = QtWidgets.QComboBox()
+        self.class_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
 
-        add_button = QtWidgets.QPushButton("添加策略")
+        add_button: QtWidgets.QPushButton = QtWidgets.QPushButton("添加策略")
         add_button.clicked.connect(self.add_strategy)
 
-        init_button = QtWidgets.QPushButton("全部初始化")
-        init_button.clicked.connect(self.strategy_engine.init_all_strategies)
+        init_button: QtWidgets.QPushButton = QtWidgets.QPushButton("全部初始化")
+        init_button.clicked.connect(self.spread_engine.init_all_strategies)
 
-        start_button = QtWidgets.QPushButton("全部启动")
-        start_button.clicked.connect(self.strategy_engine.start_all_strategies)
+        start_button: QtWidgets.QPushButton = QtWidgets.QPushButton("全部启动")
+        start_button.clicked.connect(self.spread_engine.start_all_strategies)
 
-        stop_button = QtWidgets.QPushButton("全部停止")
-        stop_button.clicked.connect(self.strategy_engine.stop_all_strategies)
+        stop_button: QtWidgets.QPushButton = QtWidgets.QPushButton("全部停止")
+        stop_button.clicked.connect(self.spread_engine.stop_all_strategies)
 
-        add_spread_button = QtWidgets.QPushButton("创建价差")
+        add_spread_button: QtWidgets.QPushButton = QtWidgets.QPushButton("创建价差")
         add_spread_button.clicked.connect(self.add_spread)
 
-        remove_spread_button = QtWidgets.QPushButton("移除价差")
+        remove_spread_button: QtWidgets.QPushButton = QtWidgets.QPushButton("移除价差")
         remove_spread_button.clicked.connect(self.remove_spread)
 
-        form = QtWidgets.QFormLayout()
+        form: QtWidgets.QFormLayout = QtWidgets.QFormLayout()
         form.addRow("价差", self.name_line)
         form.addRow("方向", self.direction_combo)
         form.addRow("价格", self.price_line)
@@ -281,7 +287,7 @@ class SpreadAlgoWidget(QtWidgets.QFrame):
         form.addRow("模式", self.mode_combo)
         form.addRow(button_start)
 
-        vbox = QtWidgets.QVBoxLayout()
+        vbox: QtWidgets.QVBoxLayout = QtWidgets.QVBoxLayout()
         vbox.addLayout(form)
         vbox.addStretch()
         vbox.addWidget(self.class_combo)
@@ -295,64 +301,84 @@ class SpreadAlgoWidget(QtWidgets.QFrame):
 
         self.setLayout(vbox)
 
-    def start_algo(self):
+    def start_algo(self) -> None:
         """"""
-        lock_str = self.mode_combo.currentText()
+        lock_str: str = self.mode_combo.currentText()
         if lock_str == "锁仓":
-            lock = True
+            lock: bool = True
         else:
-            lock = False
+            lock: bool = False
+
+        price_text = self.price_line.text()
+        volume_text = self.volume_line.text()
+        payup_text = self.payup_line.text()
+        interval_text = self.interval_line.text()
+
+        for text, name in [
+            (price_text, "价格"),
+            (volume_text, "数量"),
+            (payup_text, "超价"),
+            (interval_text, "间隔"),
+        ]:
+            if not text:
+                QtWidgets.QMessageBox.warning(
+                    self,
+                    "启动失败",
+                    f"请输入【{name}】",
+                    QtWidgets.QMessageBox.Ok
+                )
+                return
 
         self.spread_engine.start_algo(
             spread_name=self.name_line.text(),
             direction=Direction(self.direction_combo.currentText()),
-            price=float(self.price_line.text()),
-            volume=float(self.volume_line.text()),
-            payup=int(self.payup_line.text()),
-            interval=int(self.interval_line.text()),
+            price=float(price_text),
+            volume=float(volume_text),
+            payup=int(payup_text),
+            interval=int(interval_text),
             lock=lock,
             extra={}
         )
 
-    def add_spread(self):
+    def add_spread(self) -> None:
         """"""
-        dialog = SpreadDataDialog(self.spread_engine)
+        dialog: SpreadDataDialog = SpreadDataDialog(self.spread_engine)
         dialog.exec_()
 
-    def remove_spread(self):
+    def remove_spread(self) -> None:
         """"""
-        dialog = SpreadRemoveDialog(self.spread_engine)
+        dialog: SpreadRemoveDialog = SpreadRemoveDialog(self.spread_engine)
         dialog.exec_()
 
-    def update_class_combo(self):
+    def update_class_combo(self) -> None:
         """"""
         self.class_combo.clear()
         self.class_combo.addItems(
-            self.strategy_engine.get_all_strategy_class_names()
+            self.spread_engine.get_all_strategy_class_names()
         )
 
-    def remove_strategy(self, strategy_name):
+    def remove_strategy(self, strategy_name) -> None:
         """"""
         manager = self.managers.pop(strategy_name)
         manager.deleteLater()
 
-    def add_strategy(self):
+    def add_strategy(self) -> None:
         """"""
-        class_name = str(self.class_combo.currentText())
+        class_name: str = str(self.class_combo.currentText())
         if not class_name:
             return
 
-        parameters = self.strategy_engine.get_strategy_class_parameters(
+        parameters: dict = self.spread_engine.get_strategy_class_parameters(
             class_name)
-        editor = SettingEditor(parameters, class_name=class_name)
-        n = editor.exec_()
+        editor: SettingEditor = SettingEditor(parameters, class_name=class_name)
+        n: int = editor.exec_()
 
-        if n == editor.Accepted:
-            setting = editor.get_setting()
-            spread_name = setting.pop("spread_name")
-            strategy_name = setting.pop("strategy_name")
+        if n == editor.DialogCode.Accepted:
+            setting: dict = editor.get_setting()
+            spread_name: str = setting.pop("spread_name")
+            strategy_name: str = setting.pop("strategy_name")
 
-            self.strategy_engine.add_strategy(
+            self.spread_engine.add_strategy(
                 class_name, strategy_name, spread_name, setting
             )
 
@@ -360,7 +386,7 @@ class SpreadAlgoWidget(QtWidgets.QFrame):
 class SpreadRemoveDialog(QtWidgets.QDialog):
     """"""
 
-    def __init__(self, spread_engine: SpreadEngine):
+    def __init__(self, spread_engine: SpreadEngine) -> None:
         """"""
         super().__init__()
 
@@ -368,28 +394,27 @@ class SpreadRemoveDialog(QtWidgets.QDialog):
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setWindowTitle("移除价差")
         self.setMinimumWidth(300)
 
-        self.name_combo = QtWidgets.QComboBox()
-        spreads = self.spread_engine.get_all_spreads()
-        for spread in spreads:
-            self.name_combo.addItem(spread.name)
+        self.name_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
+        names: List[SpreadData] = self.spread_engine.get_all_spread_names()
+        self.name_combo.addItems(names)
 
-        button_remove = QtWidgets.QPushButton("移除")
+        button_remove: QtWidgets.QPushButton = QtWidgets.QPushButton("移除")
         button_remove.clicked.connect(self.remove_spread)
 
-        hbox = QtWidgets.QHBoxLayout()
+        hbox: QtWidgets.QHBoxLayout = QtWidgets.QHBoxLayout()
         hbox.addWidget(self.name_combo)
         hbox.addWidget(button_remove)
 
         self.setLayout(hbox)
 
-    def remove_spread(self):
+    def remove_spread(self) -> None:
         """"""
-        spread_name = self.name_combo.currentText()
+        spread_name: str = self.name_combo.currentText()
         self.spread_engine.remove_spread(spread_name)
         self.accept()
 
@@ -397,37 +422,38 @@ class SpreadRemoveDialog(QtWidgets.QDialog):
 class SpreadStrategyMonitor(QtWidgets.QWidget):
     """"""
 
-    signal_strategy = QtCore.pyqtSignal(Event)
+    signal_strategy: QtCore.Signal = QtCore.Signal(Event)
 
-    def __init__(self, spread_engine: SpreadEngine):
+    def __init__(self, main_engine: MainEngine, event_engine: EventEngine) -> None:
         super().__init__()
 
-        self.strategy_engine = spread_engine.strategy_engine
-        self.main_engine = spread_engine.main_engine
-        self.event_engine = spread_engine.event_engine
+        self.main_engine: MainEngine = main_engine
+        self.event_engine: EventEngine = event_engine
 
-        self.managers = {}
+        self.spread_engine: SpreadEngine = main_engine.get_engine(APP_NAME)
+
+        self.managers: Dict[str, SpreadStrategyWidget] = {}
 
         self.init_ui()
         self.register_event()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
-        self.scroll_layout = QtWidgets.QVBoxLayout()
+        self.scroll_layout: QtWidgets.QVBoxLayout = QtWidgets.QVBoxLayout()
         self.scroll_layout.addStretch()
 
-        scroll_widget = QtWidgets.QWidget()
+        scroll_widget: QtWidgets.QWidget = QtWidgets.QWidget()
         scroll_widget.setLayout(self.scroll_layout)
 
-        scroll_area = QtWidgets.QScrollArea()
+        scroll_area: QtWidgets.QScrollArea = QtWidgets.QScrollArea()
         scroll_area.setWidgetResizable(True)
         scroll_area.setWidget(scroll_widget)
 
-        vbox = QtWidgets.QVBoxLayout()
+        vbox: QtWidgets.QVBoxLayout = QtWidgets.QVBoxLayout()
         vbox.addWidget(scroll_area)
         self.setLayout(vbox)
 
-    def register_event(self):
+    def register_event(self) -> None:
         """"""
         self.signal_strategy.connect(self.process_strategy_event)
 
@@ -435,24 +461,24 @@ class SpreadStrategyMonitor(QtWidgets.QWidget):
             EVENT_SPREAD_STRATEGY, self.signal_strategy.emit
         )
 
-    def process_strategy_event(self, event):
+    def process_strategy_event(self, event) -> None:
         """
         Update strategy status onto its monitor.
         """
-        data = event.data
-        strategy_name = data["strategy_name"]
+        data: dict = event.data
+        strategy_name: str = data["strategy_name"]
 
         if strategy_name in self.managers:
-            manager = self.managers[strategy_name]
+            manager: SpreadStrategyWidget = self.managers[strategy_name]
             manager.update_data(data)
         else:
-            manager = SpreadStrategyWidget(self, self.strategy_engine, data)
+            manager: SpreadStrategyWidget = SpreadStrategyWidget(self, self.spread_engine, data)
             self.scroll_layout.insertWidget(0, manager)
             self.managers[strategy_name] = manager
 
-    def remove_strategy(self, strategy_name):
+    def remove_strategy(self, strategy_name) -> None:
         """"""
-        manager = self.managers.pop(strategy_name)
+        manager: SpreadStrategyWidget = self.managers.pop(strategy_name)
         manager.deleteLater()
 
 
@@ -464,104 +490,104 @@ class SpreadStrategyWidget(QtWidgets.QFrame):
     def __init__(
         self,
         strategy_monitor: SpreadStrategyMonitor,
-        strategy_engine: SpreadStrategyEngine,
+        spread_engine: SpreadEngine,
         data: dict
-    ):
+    ) -> None:
         """"""
         super().__init__()
 
-        self.strategy_monitor = strategy_monitor
-        self.strategy_engine = strategy_engine
+        self.strategy_monitor: SpreadStrategyMonitor = strategy_monitor
+        self.spread_engine: SpreadEngine = spread_engine
 
-        self.strategy_name = data["strategy_name"]
-        self._data = data
+        self.strategy_name: str = data["strategy_name"]
+        self._data: dict = data
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setFixedHeight(300)
-        self.setFrameShape(self.Box)
+        self.setFrameShape(self.Shape.Box)
         self.setLineWidth(1)
 
-        init_button = QtWidgets.QPushButton("初始化")
+        init_button: QtWidgets.QPushButton = QtWidgets.QPushButton("初始化")
         init_button.clicked.connect(self.init_strategy)
 
-        start_button = QtWidgets.QPushButton("启动")
+        start_button: QtWidgets.QPushButton = QtWidgets.QPushButton("启动")
         start_button.clicked.connect(self.start_strategy)
 
-        stop_button = QtWidgets.QPushButton("停止")
+        stop_button: QtWidgets.QPushButton = QtWidgets.QPushButton("停止")
         stop_button.clicked.connect(self.stop_strategy)
 
-        edit_button = QtWidgets.QPushButton("编辑")
+        edit_button: QtWidgets.QPushButton = QtWidgets.QPushButton("编辑")
         edit_button.clicked.connect(self.edit_strategy)
 
-        remove_button = QtWidgets.QPushButton("移除")
+        remove_button: QtWidgets.QPushButton = QtWidgets.QPushButton("移除")
         remove_button.clicked.connect(self.remove_strategy)
 
-        strategy_name = self._data["strategy_name"]
-        spread_name = self._data["spread_name"]
-        class_name = self._data["class_name"]
-        author = self._data["author"]
+        strategy_name: str = self._data["strategy_name"]
+        spread_name: str = self._data["spread_name"]
+        class_name: str = self._data["class_name"]
+        author: str = self._data["author"]
 
-        label_text = (
+        label_text: str = (
             f"{strategy_name}  -  {spread_name}  ({class_name} by {author})"
         )
-        label = QtWidgets.QLabel(label_text)
-        label.setAlignment(QtCore.Qt.AlignCenter)
+        label: QtWidgets.QLabel = QtWidgets.QLabel(label_text)
+        label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
 
-        self.parameters_monitor = StrategyDataMonitor(self._data["parameters"])
-        self.variables_monitor = StrategyDataMonitor(self._data["variables"])
+        self.parameters_monitor: StrategyDataMonitor = StrategyDataMonitor(self._data["parameters"])
+        self.variables_monitor: StrategyDataMonitor = StrategyDataMonitor(self._data["variables"])
 
-        hbox = QtWidgets.QHBoxLayout()
+        hbox: QtWidgets.QHBoxLayout = QtWidgets.QHBoxLayout()
         hbox.addWidget(init_button)
         hbox.addWidget(start_button)
         hbox.addWidget(stop_button)
         hbox.addWidget(edit_button)
         hbox.addWidget(remove_button)
 
-        vbox = QtWidgets.QVBoxLayout()
+        vbox: QtWidgets.QVBoxLayout = QtWidgets.QVBoxLayout()
         vbox.addWidget(label)
         vbox.addLayout(hbox)
         vbox.addWidget(self.parameters_monitor)
         vbox.addWidget(self.variables_monitor)
         self.setLayout(vbox)
 
-    def update_data(self, data: dict):
+    def update_data(self, data: dict) -> None:
         """"""
         self._data = data
 
         self.parameters_monitor.update_data(data["parameters"])
         self.variables_monitor.update_data(data["variables"])
 
-    def init_strategy(self):
+    def init_strategy(self) -> None:
         """"""
-        self.strategy_engine.init_strategy(self.strategy_name)
+        self.spread_engine.init_strategy(self.strategy_name)
 
-    def start_strategy(self):
+    def start_strategy(self) -> None:
         """"""
-        self.strategy_engine.start_strategy(self.strategy_name)
+        self.spread_engine.start_strategy(self.strategy_name)
 
-    def stop_strategy(self):
+    def stop_strategy(self) -> None:
         """"""
-        self.strategy_engine.stop_strategy(self.strategy_name)
+        self.spread_engine.stop_strategy(self.strategy_name)
 
-    def edit_strategy(self):
+    def edit_strategy(self) -> None:
         """"""
-        strategy_name = self._data["strategy_name"]
+        strategy_name: str = self._data["strategy_name"]
 
-        parameters = self.strategy_engine.get_strategy_parameters(
+        parameters: dict = self.spread_engine.get_strategy_parameters(
             strategy_name)
-        editor = SettingEditor(parameters, strategy_name=strategy_name)
-        n = editor.exec_()
+        editor: SettingEditor = SettingEditor(parameters, strategy_name=strategy_name)
+        n: int = editor.exec_()
 
-        if n == editor.Accepted:
-            setting = editor.get_setting()
-            self.strategy_engine.edit_strategy(strategy_name, setting)
+        if n == editor.DialogCode.Accepted:
+            setting: dict = editor.get_setting()
+            self.spread_engine.edit_strategy(strategy_name, setting)
 
-    def remove_strategy(self):
+    def remove_strategy(self) -> None:
         """"""
-        result = self.strategy_engine.remove_strategy(self.strategy_name)
+        result: bool = self.spread_engine.remove_strategy(self.strategy_name)
 
         # Only remove strategy gui manager if it has been removed from engine
         if result:
@@ -573,41 +599,41 @@ class StrategyDataMonitor(QtWidgets.QTableWidget):
     Table monitor for parameters and variables.
     """
 
-    def __init__(self, data: dict):
+    def __init__(self, data: dict) -> None:
         """"""
         super().__init__()
 
-        self._data = data
-        self.cells = {}
+        self._data: dict = data
+        self.cells: dict = {}
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
-        labels = list(self._data.keys())
+        labels: list = list(self._data.keys())
         self.setColumnCount(len(labels))
         self.setHorizontalHeaderLabels(labels)
 
         self.setRowCount(1)
         self.verticalHeader().setSectionResizeMode(
-            QtWidgets.QHeaderView.Stretch
+            QtWidgets.QHeaderView.ResizeMode.Stretch
         )
         self.verticalHeader().setVisible(False)
-        self.setEditTriggers(self.NoEditTriggers)
+        self.setEditTriggers(self.EditTrigger.NoEditTriggers)
 
         for column, name in enumerate(self._data.keys()):
             value = self._data[name]
 
-            cell = QtWidgets.QTableWidgetItem(str(value))
-            cell.setTextAlignment(QtCore.Qt.AlignCenter)
+            cell: QtWidgets.QTableWidgetItem = QtWidgets.QTableWidgetItem(str(value))
+            cell.setTextAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
 
             self.setItem(0, column, cell)
             self.cells[name] = cell
 
-    def update_data(self, data: dict):
+    def update_data(self, data: dict) -> None:
         """"""
         for name, value in data.items():
-            cell = self.cells[name]
+            cell: QtWidgets.QTableWidgetItem = self.cells[name]
             cell.setText(str(value))
 
 
@@ -618,57 +644,57 @@ class SettingEditor(QtWidgets.QDialog):
 
     def __init__(
         self, parameters: dict, strategy_name: str = "", class_name: str = ""
-    ):
+    ) -> None:
         """"""
         super(SettingEditor, self).__init__()
 
-        self.parameters = parameters
-        self.strategy_name = strategy_name
-        self.class_name = class_name
+        self.parameters: dict = parameters
+        self.strategy_name: str = strategy_name
+        self.class_name: str = class_name
 
-        self.edits = {}
+        self.edits: dict = {}
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
-        form = QtWidgets.QFormLayout()
+        form: QtWidgets.QFormLayout = QtWidgets.QFormLayout()
 
         # Add spread_name and name edit if add new strategy
         if self.class_name:
             self.setWindowTitle(f"添加策略：{self.class_name}")
-            button_text = "添加"
-            parameters = {"strategy_name": "", "spread_name": ""}
+            button_text: str = "添加"
+            parameters: dict = {"strategy_name": "", "spread_name": ""}
             parameters.update(self.parameters)
         else:
             self.setWindowTitle(f"参数编辑：{self.strategy_name}")
-            button_text = "确定"
+            button_text: str = "确定"
             parameters = self.parameters
 
         for name, value in parameters.items():
             type_ = type(value)
 
-            edit = QtWidgets.QLineEdit(str(value))
+            edit: QtWidgets.QLineEdit = QtWidgets.QLineEdit(str(value))
             if type_ is int:
-                validator = QtGui.QIntValidator()
+                validator: QtGui.QIntValidator = QtGui.QIntValidator()
                 edit.setValidator(validator)
             elif type_ is float:
-                validator = QtGui.QDoubleValidator()
+                validator: QtGui.QDoubleValidator = QtGui.QDoubleValidator()
                 edit.setValidator(validator)
 
             form.addRow(f"{name} {type_}", edit)
 
             self.edits[name] = (edit, type_)
 
-        button = QtWidgets.QPushButton(button_text)
+        button: QtWidgets.QPushButton = QtWidgets.QPushButton(button_text)
         button.clicked.connect(self.accept)
         form.addRow(button)
 
         self.setLayout(form)
 
-    def get_setting(self):
+    def get_setting(self) -> dict:
         """"""
-        setting = {}
+        setting: dict = {}
 
         if self.class_name:
             setting["class_name"] = self.class_name
@@ -679,9 +705,9 @@ class SettingEditor(QtWidgets.QDialog):
 
             if type_ == bool:
                 if value_text == "True":
-                    value = True
+                    value: bool = True
                 else:
-                    value = False
+                    value: bool = False
             else:
                 value = type_(value_text)
 
@@ -693,24 +719,24 @@ class SettingEditor(QtWidgets.QDialog):
 class SpreadDataDialog(QtWidgets.QDialog):
     """"""
 
-    def __init__(self, spread_engine: SpreadEngine):
+    def __init__(self, spread_engine: SpreadEngine) -> None:
         """"""
         super().__init__()
 
         self.spread_engine: SpreadEngine = spread_engine
 
-        self.leg_widgets = []
+        self.leg_widgets: list = []
 
         self.init_ui()
 
-    def init_ui(self):
+    def init_ui(self) -> None:
         """"""
         self.setWindowTitle("创建价差")
 
-        self.name_line = QtWidgets.QLineEdit()
-        self.active_line = QtWidgets.QLineEdit()
+        self.name_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
+        self.active_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
 
-        self.min_volume_combo = QtWidgets.QComboBox()
+        self.min_volume_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
         self.min_volume_combo.addItems([
             "1",
             "0.1",
@@ -721,16 +747,16 @@ class SpreadDataDialog(QtWidgets.QDialog):
             "0.000001",
         ])
 
-        self.formula_line = QtWidgets.QLineEdit()
+        self.formula_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
 
-        self.grid = QtWidgets.QGridLayout()
+        self.grid: QtWidgets.QGridLayout = QtWidgets.QGridLayout()
 
-        button_add = QtWidgets.QPushButton("创建价差")
+        button_add: QtWidgets.QPushButton = QtWidgets.QPushButton("创建价差")
         button_add.clicked.connect(self.add_spread)
 
-        Label = QtWidgets.QLabel
+        Label: QtWidgets.QLabel = QtWidgets.QLabel
 
-        grid = QtWidgets.QGridLayout()
+        grid: QtWidgets.QGridLayout = QtWidgets.QGridLayout()
         grid.addWidget(Label("价差名称"), 0, 0)
         grid.addWidget(self.name_line, 0, 1, 1, 4)
         grid.addWidget(Label("主动腿代码"), 1, 0)
@@ -744,18 +770,18 @@ class SpreadDataDialog(QtWidgets.QDialog):
         grid.addWidget(Label("交易方向"), 4, 2)
         grid.addWidget(Label("交易乘数"), 4, 3)
 
-        int_validator = QtGui.QIntValidator()
+        int_validator: QtGui.QIntValidator = QtGui.QIntValidator()
         int_validator.setBottom(0)
 
-        leg_count = 5
-        variables = ["A", "B", "C", "D", "E"]
+        leg_count: int = 5
+        variables: list = ["A", "B", "C", "D", "E"]
         for i, variable in enumerate(variables):
-            symbol_line = QtWidgets.QLineEdit()
+            symbol_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
 
-            direction_combo = QtWidgets.QComboBox()
+            direction_combo: QtWidgets.QComboBox = QtWidgets.QComboBox()
             direction_combo.addItems(["买入", "卖出"])
 
-            trading_line = QtWidgets.QLineEdit()
+            trading_line: QtWidgets.QLineEdit = QtWidgets.QLineEdit()
             trading_line.setValidator(int_validator)
 
             grid.addWidget(Label(variable), 5 + i, 0)
@@ -763,7 +789,7 @@ class SpreadDataDialog(QtWidgets.QDialog):
             grid.addWidget(direction_combo, 5 + i, 2)
             grid.addWidget(trading_line, 5 + i, 3)
 
-            d = {
+            d: dict = {
                 "variable": variable,
                 "symbol": symbol_line,
                 "direction": direction_combo,
@@ -776,9 +802,9 @@ class SpreadDataDialog(QtWidgets.QDialog):
 
         self.setLayout(grid)
 
-    def add_spread(self):
+    def add_spread(self) -> None:
         """"""
-        spread_name = self.name_line.text()
+        spread_name: str = self.name_line.text()
         if not spread_name:
             QtWidgets.QMessageBox.warning(
                 self,
@@ -788,7 +814,7 @@ class SpreadDataDialog(QtWidgets.QDialog):
             )
             return
 
-        price_formula = self.formula_line.text()
+        price_formula: str = self.formula_line.text()
         if not self.check_formula(price_formula):
             QtWidgets.QMessageBox.warning(
                 self,
@@ -798,20 +824,20 @@ class SpreadDataDialog(QtWidgets.QDialog):
             )
             return
 
-        active_symbol = self.active_line.text()
-        min_volume = float(self.min_volume_combo.currentText())
+        active_symbol: str = self.active_line.text()
+        min_volume: str = float(self.min_volume_combo.currentText())
 
-        leg_settings = {}
+        leg_settings: dict = {}
         for d in self.leg_widgets:
             try:
-                vt_symbol = d["symbol"].text()
-                trading_multiplier = int(d["trading"].text())
+                vt_symbol: str = d["symbol"].text()
+                trading_multiplier: int = int(d["trading"].text())
 
                 if d["direction"].currentText() == "买入":
-                    trading_direction = 1
+                    trading_direction: int = 1
                 else:
-                    trading_direction = -1
-                trading_multiplier = trading_multiplier * trading_direction
+                    trading_direction: int = -1
+                trading_multiplier: int = trading_multiplier * trading_direction
 
                 leg_settings[vt_symbol] = {
                     "variable": d["variable"],
@@ -849,12 +875,12 @@ class SpreadDataDialog(QtWidgets.QDialog):
         )
         self.accept()
 
-    def check_formula(self, formula: str):
+    def check_formula(self, formula: str) -> bool:
         """"""
-        data = {variable: 1 for variable in "ABCDE"}
+        data: dict = {variable: 1 for variable in "ABCDE"}
         locals().update(data)
         try:
-            result = eval(formula)
+            result: Any = eval(formula)
             return True
         except Exception:
             return False
